<?php
namespace Teknisa\MAN\Services; 

use Doctrine\ORM\EntityManager;
use Geral\Entities\Operador;
use Zeedhi\Framework\Session\Session;
use Teknisa\MAN\Utilities;

class Login {

    const SITUACAO_ATIVO = 'A';

    protected $entityManager;
    protected $criptografia;
    protected $operador;
    protected $session;
    /** @var Environment */
    
    public function __construct(EntityManager $entityManager, Criptografia $criptografia, Session $session) {
        $this->entityManager = $entityManager;
        $this->criptografia = $criptografia;
        $this->session = $session;
    }

    private function atualizaSenhaDoOperador($senhaInformada) {
        $senhaCriptografadaWeb = $this->criptografia->retornaSenhaWeb($senhaInformada);
        $this->operador->setCdsenhaoperweb($senhaCriptografadaWeb);
        $this->entityManager->persist($this->operador);
        $this->entityManager->flush();
    }

    private function validaSenhaDelphi($senhaDelphi, $senhaInformada) {
        $senhaCriptografadaDelphi = $this->criptografia->retornaSenhaDelphi($senhaInformada);
        if($senhaDelphi != $senhaCriptografadaDelphi) {
            throw new \Exception("Senha inválida.");
        }

        $this->atualizaSenhaDoOperador($senhaInformada);
    }

    private function validaSenhaWeb($senhaWeb, $senhaInformada) {
        $senhaCriptografadaWeb = $this->criptografia->retornaSenhaWeb($senhaInformada);
        if($senhaWeb != $senhaCriptografadaWeb) {
            throw new \Exception("Senha inválida.");
        }
    }

    private function validaSenhas($senhaDelphi, $senhaWeb, $senhaInformada) {
        $this->validaSenhaWeb($senhaWeb, $senhaInformada);
       
    }

    private function validaParametros($operador) {
        // verifica situação do operador
        if($operador->getIdoperativo() !== self::SITUACAO_ATIVO) {
            throw new \Exception("Operador não está ativo.");
        }

        // verifica estrutura de trabalho do operador
        if(!$operador->getNrestruttrab()) {
            throw new \Exception("Operador não possui estrutura de trabalho.");
        }

        // verifica a organização do operador
        if(!$operador->getNrorgtrab()) {
            throw new \Exception("Operador não possui organização de trabalho.");
        }
    }

    private function retornaOperadorPorEmail($email) {
		$param = array('DSEMAILOPER'=>$email);
		$stm = $this->entityManager->getConnection()->prepare(Utilities\Query::GET_OPERADOR);
		$stm->execute($param);
		$operador = $stm->fetchAll();
        if (!$operador) {
            throw new \Exception("Operador não está cadastrado.");
        }
        return $operador;
    }

    private function retornaParametrosFilial($operador, $unidadeTrabalho) {
        return $this->entityManager->getRepository('\Geral\Entities\Paramfilial')
            ->findOneBy(array("nrorg" => $operador->getNrorgtrab(), "cdfilial" => $unidadeTrabalho));
    }
  
    private function factoryDadosDaSessao($operador) {
        return array(
            "cdoperador" => $operador[0]['CDOPERADOR'],
            "ididioma" => $operador[0]['IDIDIOMA'],
            "nmoperador" => $operador[0]['NMOPERADOR'],
            "nrorgtrab" => $operador[0]['NRORGTRAB'],
            "nrorg" => $operador[0]['NRORG'],
            "email" => $operador[0]['DSEMAILOPER'],
            "senhacryp" => $operador[0]['CDSENHAOPERWEB']
        );
    }

    public function autenticar($email, $senha) {
        $operador = $this->retornaOperadorPorEmail($email);
        $cdsenhoper = $operador[0]['CDSENHOPER'];
        $cdsenhaoperweb = $operador[0]['CDSENHAOPERWEB'];
        $this->validaSenhas($cdsenhoper, $cdsenhaoperweb, $senha);
        if ($operador[0]['NRORGTRAB']){
            $this->session->set("NRORGTRAB", $operador[0]['NRORGTRAB']);
            $this->session->set("NRORG", $operador[0]['NRORGTRAB']);
        }
        else {
            $this->session->set("NRORGTRAB", $operador[0]['NRORG']);
            $this->session->set("NRORG", $operador[0]['NRORG']);
        }
        if ($operador[0]['CDOPERADOR']){
            $this->session->set("CDOPERADOR", $operador[0]['CDOPERADOR']);
        }
        $dadosDaSessao = $this->factoryDadosDaSessao($operador);
        
        return $dadosDaSessao;
    }

    public function sair(){
        try {
            $this->session->clear();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function pegaArquivoVersao(){
        $strFile  = file_get_contents($this->arquivoVersao);
        $jsonFile = json_decode($strFile, true);

        return array(
            'VERSION'  => $jsonFile['version'],
            'REVISION' => $jsonFile['revision'],
            'BUILD'    => $jsonFile['build']
        );
    }

    public function habilitaLogin($cdoperador, $senhacryp){
        $operador = $this->retornaDadosPorOperador($cdoperador);
        $cdsenhaoperweb = $operador[0]['CDSENHAOPERWEB'];
        $this->validaSenhasAutoLogin($cdsenhaoperweb, $senhacryp);
        $dadosDaSessao = $this->factoryDadosDaSessao($operador);
        
        return $dadosDaSessao;
    }

    private function retornaDadosPorOperador($cdoperador) {
        $param = array('CDOPERADOR'=>$cdoperador);
        $stm = $this->entityManager->getConnection()->prepare(Utilities\Query::GET_OPERADOR_BY_CDOPERADOR);
        $stm->execute($param);
        $operador = $stm->fetchAll();
        if (!$operador) {
            throw new \Exception("Operador não está cadastrado.");
        }
        return $operador;
    }

    private function validaSenhasAutoLogin($senhaWeb, $senhaInformada) {
        if($senhaWeb != $senhaInformada) {
            throw new \Exception("Senha inválida.");
        }       
    }

}
